<?php

class CalendarHolder extends Page{

	static $many_many = array(
      'Events' => 'Event'
	);
	static $has_one = array(
		"Sidebar" => "WidgetArea",
    );
	
	/*
	 * Get the Fields needed by the CMS back end
	*/
	function getCMSFields(){

		$fields = parent::getCMSFields();
		$tablefield = new ManyManyComplexTableField(
			$this,
			'Events',
			'Event',
			array(
				'Start' => 'Start',
				'End' => 'End',
				'Title' => 'Title',
			),
			'getCMSFields_forPopup'
		);
		$tablefield->setAddTitle( 'Event' );

		$fields->addFieldToTab( 'Root.Content.Events', $tablefield );
		if(!$fields->dataFieldByName("Sidebar")) {
			$fields->addFieldToTab("Root.Content.Widgets", new WidgetAreaEditor("Sidebar"));
		}
		return $fields;	
	}
	
	/*
	 * Create a first Calendar while running the installation
	*/
	function requireDefaultRecords() {
		
		parent::requireDefaultRecords();
		
		if(!DataObject::get_one('CalendarHolder')) {
		
			$calendarholder = new CalendarHolder();
			$calendarholder->Title = "Calendar";
			$calendarholder->URLSegment = "calendar";
			$calendarholder->Status = "Published";
			$calendarholder->write();
			$calendarholder->publish("Stage", "Live");
		}
	}
	
}

class CalendarHolder_Controller extends Page_Controller{
	
	/*
	 * The initializing function, this calls all the CSS and the Javascripts which the module needs
	*/
	function init(){
	
		parent::init();	
		if(Director::fileExists(project() . "/css/calendar.css")) {
	    	Requirements::css(project() . "/css/calendar.css");
		}
		else{
        	Requirements::css("calendar/css/calendar.css");
     	}
		
	}
	/*
	 * Get the link to the previous month
	*/
	function getPreviousMonth(){
		// get todays date.
		$today = getdate();
		//if the date is not today
		if(isset($_GET['month'])){
			$today = getdate((int)$_GET['month']);
		}
		$lastmonth = getdate( mktime(0,0,0,$today['mon'] - 1 ,1,$today['year']) );
		$output = new DataObjectSet();
		$output->push(
			new ArrayData(
				array(
					"Link" => $this->URLSegment . "/?month=" . $lastmonth[0],
					"Name" => $lastmonth['month']
				)
			)
		);
		return $output;
	}
	
	/*
	 * Get the link to the next month
	*/
	function getNextMonth(){
		// get todays date.
		$today = getdate();
		//if the date is not today
		if(isset($_GET['month'])){
			$today = getdate((int)$_GET['month']);
		}
		$lastmonth = getdate( mktime(0,0,0,$today['mon'] + 1 ,1,$today['year']) );
		$output = new DataObjectSet();
		$output->push(
			new ArrayData(
				array(
					"Link" => $this->URLSegment . "/?month=" . $lastmonth[0],
					"Name" => $lastmonth['month']
				)
			)
		);
		return $output;
	}
	
	
	/*
	 * Get the exact month's calendar
	*/
	function getCalendar(){
		// get todays date.
		$today = getdate();
		//if the date is not today
		if(isset($_GET['month'])){
			$today = getdate((int)$_GET['month']);
		}
		$firstday = getdate( mktime(0,0,0,$today['mon'],1,$today['year']) );
		$lastday = getdate( mktime(0,0,0,$today['mon'] + 1,0,$today['year']) );
		$output = new DataObjectSet();
		$days = new DataObjectSet();
		
		for($i = 0; $i < ($lastday['mday'] - $firstday['mday'] + 1); $i++){
			
			$oddeven = "odd";
			$oddeven = ($i % 2 == 0) ? "odd" : "even";
			
			$thisday = getdate(mktime(0,0,0,$today['mon'],$i + 1,$today['year']));
			
			$today_to_search = $thisday['year'] . "-" . $thisday['mon'] . "-" . $thisday['mday'];
			
			//get the event is type is not choosen
			$events = DataObject::get("Event","CalendarHolderID = '" . $this->ID ."' AND (Start = '$today_to_search' OR End = '$today_to_search' OR '$today_to_search' BETWEEN Start AND End)", "StartTime", "JOIN CalendarHolder_Events ON `Event`.ID = CalendarHolder_Events.EventID");
			//get the events if the type is choosen
			if(strcmp(Session::get("EventType"),"all") && strcmp(Session::get("EventType"),"") ){
				$events = DataObject::get("Event","CalendarHolderID = '" . $this->ID ."' AND (Start = '$today_to_search' OR End = '$today_to_search' OR '$today_to_search' BETWEEN Start AND End) AND EventType = '" . Session::get("EventType") ."'", "StartTime", "JOIN CalendarHolder_Events ON `Event`.ID = CalendarHolder_Events.EventID");
			}
			
			$days->push(
				new ArrayData(
					array(
						"Date" => ($i + 1),
						"Day" => substr($thisday['weekday'],0,3),
						"Mon" => substr($thisday['month'],0,3),
						"Year" => $today['year'],
						"oddeven" => $oddeven,
						"Events" => $events,
						"Epoch" => $thisday[0]
					)
				)
			);
		}
		$output->push(
			new ArrayData(
				array(
					"Month" => $today['month'],
					"Year" => $today['year'],
					"Days" => $days
				)
			)
		);
		
		return $output;
		
	}
	
	/*
	 * Delete an entry
	*/
	function delete(){
		if(isset($_GET['id']))
			DataObject::delete_by_id("Event",(int)$_GET['id']);
		Director::redirectback();
		
	}
	
	/*
	 * Get entry by an id
	*/
	function getEvent(){
		return DataObject::get_by_id("Event",(int)$_GET['id']);
	}
	
	/*
	 * Edit an ad
	*/
	function edit(){
		return array(
			"Title" => "Edit an event",
			"Form" => $this->EditEventForm()
		);
	}
	
	/*
	 * Edit Event Form this function returns a form which is used to edit a specific event
	*/
	function EditEventForm(){
		$event = $this->getEvent();

		return new Form($this, "EditEventForm",
			new FieldSet(
				new TextField("Title", "Title", ($event->Title)
												? $event->Title
												: "" ),
				new TextField("EventType", "Event Type", $value = $event->EventType),
				new CalendarDateField('Start', 'Starting date'),
				new CalendarDateField('End', 'Ending date'),
				new TimeField('StartTime', 'Start Time', $value = ($event->StartTime)
														? $event->StartTime
														: "" 
														,$timeformat = "g:ia"
														),
				new TimeField('EndTime', 'EndTime', $value = ($event->EndTime)
														? $event->EndTime
														: "" 
														,$timeformat = "g:ia"
														),
				new TextField("Location", "Location", $value = $event->Location),
				new TextareaField("EventContent", "Content", 4, 5, $value = $event->Content),
				new HiddenField("ID", "ID", ($event->ID)
													? $event->ID
													: "" )
			),
			new FieldSet(
				new FormAction("editAnEvent", "Edit")
			),
			new RequiredFields("Title", "Start", "End"));

	}
	
	/*
	 * Edit event form processing
	*/
	function editAnEvent($data, $form){
		$event = DataObject::get_by_id("Event",(int)$data['ID']);
		
		if($event){
			echo $data['Start'];
			
			$start = explode("/",$data['Start']);
			$start = implode("-",array($start[2], $start[1], $start[0]));
			
			$end = explode("/",$data['End']);
			$end = implode("-",array($end[2], $end[1], $end[0]));
			
			$event->Title = $data['Title'];
			$event->Start = $start;
			$event->End = $end;
			$event->StartTime = $data['StartTime'];
			$event->EndTime = $data['EndTime'];
			$event->Location = $data['Location'];
			$event->EventType = $data['EventType'];
			$event->Content = $data['EventContent'];
			
			$event->write();
			
			Director::redirectback();
			
		}
		else{
			return "Sorry! This event cannot be found in the database any more.";
		}
	}
	
	/*
	 * View a calendar event
	*/
	
	function view(){
		return array(
			"Title" => "View Event"
		);
	}
	
	/*
	 * view event type
	*/
	function type(){
		Session::set("EventType",$_GET['name']);
		$cal = DataObject::get_one('CalendarHolder');
		Director::redirect($cal->URLSegment . "/");
	}
	
	/*
	 * Set the month for the calendar widget
	 */
	function setMonth(){
		Session::set("CalMonth",(int)$_GET[month]);
		Director::redirect($_GET['back']."/");
	}
}
?>
